@@ -1,5 +1,4 @@
 #nhaneS - retrieve data from the CDC NHANES repository
-
 nhanesURL <- 'http://wwwn.cdc.gov/Nchs/Nhanes/'
 
 # Create a list of nhanes groups
@@ -34,6 +33,8 @@ nh_years['2009'] <- "2009-2010"
 nh_years['2010'] <- "2009-2010"
 nh_years['2011'] <- "2011-2012"
 nh_years['2012'] <- "2011-2012"
+#nh_years['2013'] <- "2013-2014"
+#nh_years['2014'] <- "2013-2014"
 
 # Continuous NHANES table names have a letter suffix that indicates the collection interval
 data_idx <- list()
@@ -44,6 +45,7 @@ data_idx['2005-2006'] <- "D"
 data_idx['2007-2008'] <- "E"
 data_idx['2009-2010'] <- "F"
 data_idx['2011-2012'] <- "G"
+#data_idx['2013-2014'] <- "H"
 
 
 # An internal function that converts a year into the nhanes interval.
@@ -53,6 +55,20 @@ data_idx['2011-2012'] <- "G"
 # @return The 2-year interval that includes the year, e.g. 2001-2002
 # 
 
+#------------------------------------------------------------------------------
+get_year_from_nh_table <- function(nh_table) {
+nhloc <- data.frame(stringr::str_locate_all(nh_table, '_'))
+if(nrow(nhloc)!=0){
+  if((nhloc$start[nrow(nhloc)]+1) == nchar(nh_table)) {
+    idx <- str_sub(nh_table, -1, -1) 
+    nh_year <- names(data_idx[grep(idx, data_idx)])
+  } else { ## It appears a mistake was made in the table name
+    message('Invalid column name')
+    return(NULL) }
+} else 
+  nh_year <- "1999-2000"
+#} else { nh_year <- names(data_idx[1])}
+}
 #------------------------------------------------------------------------------
 
 get_nh_survey_years <- function(year) {
@@ -65,18 +81,25 @@ get_nh_survey_years <- function(year) {
   }
 }
 
+xpath <- '//*[@id="ContentPlaceHolder1_GridView1"]'
+
 #------------------------------------------------------------------------------
 
-#' Returns a list of table names for the specified survey group
+#' Returns a list of table names for the specified survey group.
 #' 
-#' @importFrom stringr str_replace str_c str_match
-#' @param nh_surveygroup The type of survey (DEMOGRAPHIC, DIETARY, EXAMINATION, LABORATORY, QUESTIONNAIRE)
-#' @param year The year in yyyy format where 1999 <= yyyy <= 2012
-#' @param details If true then a more detailed description of the tables is returned
-#' @return The names of the tables in the specified survey group
-#' @details The first step in browsing NHANES data is to display the table names. 
-#' The table names are stored internal to the nhanes package, i.e. the information is retrieved from 
-#' internal tables such as vars_EXAMINATION_2005-2006. 
+#' Enables quick display of all available tables in the survey group.
+#' 
+#' @importFrom stringr str_replace str_c str_match str_to_title
+#' @importFrom rvest html html_nodes html_table
+#' @importFrom magrittr %>%
+#' @param nh_surveygroup The type of survey (DEMOGRAPHIC, DIETARY, EXAMINATION, LABORATORY, QUESTIONNAIRE).
+#' Abbreviated terms may also be used: (DEMO, DIET, EXAM, LAB, Q).
+#' @param year The year in yyyy format where 1999 <= yyyy <= 2012.
+#' @param details If true then a more detailed description of the tables is returned.
+#' @return The names of the tables in the specified survey group.
+#' @details Data are retrieved via web scraping using html wrappers from package rvest.
+#' It is often useful to display the table names in an NHANES survey. In effect this
+#' is a convenient way to browse the available NHANES tables.
 #' @examples
 #' nhanesTables('EXAM', 2007)
 #' nhanesTables('LAB', 2009, details=TRUE)
@@ -88,12 +111,14 @@ nhanesTables <- function(nh_surveygroup, year, details = FALSE) {
     return(NULL)
   }
   
-  #cat(year)
   nh_year <- get_nh_survey_years(year)
   
-  vartablename <- str_c('vars_', nhanes_group[[nh_surveygroup]], '_', str_replace(get_nh_survey_years(year),'-','_'), collapse='')
-  df <- do.call(data.frame, list(as.name(vartablename)) )
-
+  turl <- str_c(nhanesURL, 'search/variablelist.aspx?Component=', 
+                str_to_title(as.character(nhanes_group[nh_surveygroup])), 
+                '&CycleBeginYear=', unlist(strsplit(as.character(nh_year), '-'))[[1]] , sep='')
+  df <- as.data.frame(turl %>% html() %>% html_nodes(xpath=xpath) %>% html_table())
+  
+  
   idx <- str_c('_', data_idx[[nh_year]], sep='')
   if( idx == '_') {idx = '' }
   
@@ -120,34 +145,41 @@ nhanesTables <- function(nh_surveygroup, year, details = FALSE) {
 
 #' Displays a list of variables in the specified NHANES table.
 #' 
-#' If truncated = TRUE then only the field names and field descriptions are returned.
+#' Enables quick display of table variables and their definitions.
 #' 
 #' @importFrom stringr str_replace str_c str_sub
-#' @param nh_surveygroup The type of survey (DEMOGRAPHIC, DIETARY, EXAMINATION, LABORATORY, QUESTIONNAIRE)
-#' @param year The year in yyyy format where 1999 <= yyyy <= 2012
-#' @param nh_table The name of the specific table to retrieve
-#' @param truncated If true then only the variable names and descriptions are returned, which is often sufficient
+#' @importFrom rvest html html_nodes html_table
+#' @importFrom magrittr %>%
+#' @param nh_surveygroup The type of survey (DEMOGRAPHIC, DIETARY, EXAMINATION, LABORATORY, QUESTIONNAIRE).
+#' Abbreviated terms may also be used: (DEMO, DIET, EXAM, LAB, Q).
+#' @param nh_table The name of the specific table to retrieve.
+#' @param truncated If true then only the variable names and descriptions are returned, which is often sufficient.
 #' @param nchar The number of characters in the Variable Description to print. Values are limited to 0<=nchar<=127.
 #' This is used to enhance readability, cause variable descriptions can be very long.
 #' @return The names of the tables in the specified survey group
-#' @details Each data table contains multiple, sometimes more than 100, fields. It is helpful to list the field
+#' @details Data are retrieved via web scraping using html wrappers from package rvest.
+#' Each data table contains multiple, sometimes more than 100, fields. It is helpful to list the field
 #' descriptions to ascertain quickly if a data table is of interest.
 #' @examples
-#' nhanesTableVars('EXAM', 2007, 'BPX_E')
-#' nhanesTableVars('EXAM', 2007, 'OHX_E', truncated=TRUE)
+#' nhanesTableVars('LAB', 'CBC_E')
+#' nhanesTableVars('EXAM', 'OHX_E', truncated=TRUE)
 #' @export
 #' 
-nhanesTableVars <- function(nh_surveygroup, year, nh_table, truncated = FALSE, nchar=100) {
+nhanesTableVars <- function(nh_surveygroup, nh_table, truncated = FALSE, nchar=100) {
   if( !(nh_surveygroup %in% names(nhanes_group)) ) {
     stop("Invalid survey group")
     return(NULL)
   }
   
-  tablename <- str_c('vars_', nhanes_group[[nh_surveygroup]], '_', str_replace(get_nh_survey_years(year),'-','_'), collapse='')
-  df <- do.call(data.frame, list(as.name(tablename)) )
+  nh_year <- get_year_from_nh_table(nh_table)
+#  nh_year <- get_nh_survey_years(year)
+  turl <- str_c(nhanesURL, 'search/variablelist.aspx?Component=', 
+                str_to_title(as.character(nhanes_group[nh_surveygroup])), 
+                '&CycleBeginYear=', unlist(strsplit(as.character(nh_year), '-'))[[1]] , sep='')
+  df <- as.data.frame(turl %>% html() %>% html_nodes(xpath=xpath) %>% html_table())
   
   if(!(nh_table %in% df$Data.File.Name)) {
-    stop('Table ', nh_table, ' not present in the ', year, ' ', nh_surveygroup, ' survey' )
+    stop('Table ', nh_table, ' not present in the ', nh_surveygroup, ' survey' )
     return(NULL)
   }
   
@@ -169,29 +201,33 @@ nhanesTableVars <- function(nh_surveygroup, year, nh_table, truncated = FALSE, n
 
 #------------------------------------------------------------------------------
 
-#' Download an XPT table and return as a data frame
+#' Download an NHANES table and return as a data frame.
+#' 
+#' Use to download NHANES data tables that are in SAS format.
+#' 
 #' @importFrom Hmisc sasxport.get
 #' @importFrom stringr str_c
-#' @param year The year in yyyy format where 1999 <= yyyy <= 2012
-#' @param nh_table The name of the specific table to retrieve
-#' @return The table is returned as a data frame
-#' @details Currently, a table is downloaded in its entirety. Other than nhanesAttr, this is the
-#' only function that actually accesses data from the nhanes site directly.
+#' @param nh_table The name of the specific table to retrieve.
+#' @return The table is returned as a data frame.
+#' @details Downloads a table from the NHANES website in its entirety. NHANES tables 
+#' are stored in SAS '.XPT' format. Function nhanes uses sasxport.get from package H
+#' misc to retrieve the data.
 #' @examples 
-#' nhanes(2007, 'BPX_E')
-#' nhanes(2009, 'FOLATE_F')
+#' nhanes('BPX_E')
+#' nhanes('FOLATE_F')
 #' @export
 #' 
-nhanes <- function(year, nh_table) {
+nhanes <- function(nh_table) {
   nht <- tryCatch({    
-    url <- str_c(nhanesURL, get_nh_survey_years(year), '/', nh_table, '.XPT', collapse='')
+    nh_year <- get_year_from_nh_table(nh_table)
+#    nh_year <- names(data_idx[1])
+    url <- str_c(nhanesURL, nh_year, '/', nh_table, '.XPT', collapse='')
     return(sasxport.get(url))
-#    return(read.xport(url))
   },
   error = function(cond) {
-    message(paste("Data set ", nh_table, " from year", year, "is not available"))
-    message(url)
-    return(NA)
+    message(paste("Data set ", nh_table,  " is not available"), collapse='')
+#    message(url)
+    return(NULL)
   },
   warning = function(cond) {
     message(cond, '\n')    
@@ -202,11 +238,14 @@ nhanes <- function(year, nh_table) {
 
 #------------------------------------------------------------------------------
 
-#' Returns the attributes of an NHANES data table
+#' Returns the attributes of an NHANES data table.
+#' 
+#' Returns attributes such as number of rows, columns, and memory size,
+#' but does not reutrn the table itself.
+#' 
 #' @importFrom Hmisc sasxport.get
 #' @importFrom stringr str_c
 #' @importFrom utils object.size
-#' @param year  The year in yyyy format where 1999 <= yyyy <= 2012
 #' @param nh_table The name of the specific table to retrieve
 #' @return The following attributes are returned as a list \cr
 #' nrow = number of rows \cr
@@ -220,14 +259,15 @@ nhanes <- function(year, nh_table) {
 #' before importing into R. To retrieve these characteristics, the specified table is downloaded,
 #' characteristics are determined, then the table is deleted.
 #' @examples 
-#' nhanesAttr(2007, 'BPX_E')
-#' nhanesAttr(2009, 'FOLATE_F')
+#' nhanesAttr('BPX_E')
+#' nhanesAttr('FOLATE_F')
 #' @export
 #' 
-nhanesAttr <- function(year, nh_table) {
+nhanesAttr <- function(nh_table) {
   nht <- tryCatch({    
-    url <- str_c(nhanesURL, get_nh_survey_years(year), '/', nh_table, '.XPT', collapse='')
-#    tmp <- read.xport(url)
+    nh_year <- get_year_from_nh_table(nh_table)
+    url <- str_c(nhanesURL, nh_year, '/', nh_table, '.XPT', collapse='')
+#    url <- str_c(nhanesURL, get_nh_survey_years(year), '/', nh_table, '.XPT', collapse='')
     tmp <- sasxport.get(url)
     nhtatt <- attributes(tmp)
     nhtatt$row.names <- NULL
@@ -241,7 +281,7 @@ nhanesAttr <- function(year, nh_table) {
     return(nhtatt)
   },
   error = function(cond) {
-    message(paste("Data from year", year, "are not available"))
+    message(paste("Data from ", nh_table, " are not available"))
     return(NA)
   },
   warning = function(cond) {
@@ -253,53 +293,50 @@ nhanesAttr <- function(year, nh_table) {
 
 #------------------------------------------------------------------------------
 
-#' Translates code to it's text value
+#' Display code translation information for the specified table.
 #' 
-#' Many of the NHANES data tables have encoded values. E.g. 1 = 'Male', 2 = 'Female'
-#' We want to access the code as needed either by simply displaying the code translation,
-#' or applying the translation directly to the table. E.g, for gender we may want to
-#' translate the code (1,2) to the represented values (Male, Female).
+#' Returns code translations which is especially useful for categorical tables, 
+#' which includes most NHANES tables. 
 #' 
-#' Any coded column will have a specific data code, and many columns may share the same code translation.
-#' @param nh_surveygroup The survey group that contains the columns to be translated.
+#' @importFrom stringr str_c str_locate str_sub 
+#' @importFrom rvest html html_nodes html_table
+#' @param nh_table The name of the specific table to retrieve.
 #' @param colname The name of the column to translate
+#' @param details If TRUE. then all available table translation information is displayed.
 #' @return The code translation
-#' @details Code translations are done internally in the nhanes package. Specifically, the translations
-#' are indicated in the nhanesCodeMapXXXXX lists. 
+#' @details Code translation tables are retrieved via webscraping using rvest. 
+#' Many of the NHANES data tables have encoded values. E.g. 1 = 'Male', 2 = 'Female'.
+#' By default, the codes and the translated values are returned in a data frame.
 #' @examples
-#' nhanesTranslate('DEMO', 'DMQADFC')
-#' nhanesTranslate('LAB', 'DCD030')
+#' nhanesTranslate('DEMO_B', 'DMDBORN')
+#' nhanesTranslate('BPX_F', 'BPACSZ', details=TRUE)
 #' @export
 #' 
-nhanesTranslate <- function(nh_surveygroup, colname) {
+nhanesTranslate <- function(nh_table, colname, details=FALSE) {
   if(is.null(colname)) {
     message('Column name is required')
     return(0)
   }
   
-  switch(nh_surveygroup,
-         'DEMO' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapDemographics[[colname]]]],
-         'DEMOGRAPHICS' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapDemographics[[colname]]]],
-         'DIET' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapDietary[[colname]]]],
-         'DIETARY' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapDietary[[colname]]]],
-         'EXAM' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapExamination[[colname]]]],
-         'EXAMINATION' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapExamination[[colname]]]],
-         'LAB' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapLaboratory[[colname]]]],
-         'LABORATORY' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapLaboratory[[colname]]]],
-         'Q' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapQuestionnaire[[colname]]]],
-         'QUESTIONNAIRE' = NHANEScolumn <- nhanesTranslator[[nhanesCodeMapQuestionnaire[[colname]]]],
-         stop("Invalid survey group")
-  )
+  # Parse nh_table to find the suffix, e.g. for table 'BPX_E', the suffix is '_E'
+  # If there is no suffix, then we are likely dealing with data from 1999-2000
+  # Will use data_idx to perform an inverse match
   
-  if(is.null(NHANEScolumn)) {
-    message('Column is either not present or does not need translation. Returning NULL')
-    return(NHANEScolumn)
+  nc <- nchar(nh_table)
+  if(nc > 12) {
+    message('Column name is longer than expected')
+    return(NULL)
+  }  
+  
+  nh_year <- get_year_from_nh_table(nh_table)
+  url <- str_c(nhanesURL, nh_year, '/', nh_table, '.htm', sep='')
+  xpt <- str_c('//*[h3[a[@name="', colname, '"]]]', sep='')
+  tabletree <- url %>% html() %>% html_nodes(xpath=xpt)
+  tabletrans <- as.data.frame(html_nodes(tabletree, 'table') %>% html_table())
+  if(details == FALSE) {
+    tabletrans <- tabletrans[,c('Code.or.Value', 'Value.Description')]
   }
-  else {
-    translated <- as.data.frame(unlist(NHANEScolumn))
-    names(translated) <- 'Translation'
-    return(translated)
-  } 
+  return(tabletrans)
 }
 
 #------------------------------------------------------------------------------
